// cristinel.ababei, copyleft :-), 2019-present
//
// this is a simple project that demonstrates the use of of the WiFi module
// ESP-01 (uses the popular chip ESP8266); it essentially is the example 1
// in Lab #9 or COEN-4720 at Marquette;
// -- it uses UART2 to connect serial terminal on host PC; for printing stuff and debugging purposes;
// -- it uses UART1 to connect to the ESP8266 module; the connections to the wifi module are:
//  Nucleo Board        -> WiFi Module
//  PA9 (TX of UART1)   -> RX of WiFi module
//  PA10 (RX of UART1)  -> TX of WiFi module
//  GND from Nucleo     -> GND of WiFi module
//  3.3V from Nucleo    -> VCC of WiFi module
//  3.3V from Nucleo    -> CH_PD of WiFi module
//
// commands "effect=1" and "effect=2" are sent from the Android App;
// these commands are received by the wifi module and used by this program
// running on NUcleo board to tunr the LED ON or OFF;
// notes:
// -- UART2 uses 115200 and UART1 uses 9600 baudrates
// -- that is why first you will need to set the baudrate of your wifi module
// to 9600 using an off-the-shelf USB-to-TTL converter; you have it in the lab kit;
// you designed one in KiCAD and will solder it later; one can also buy from Amazon:
// https://www.amazon.com/gp/product/B00N4MCS1A/ref=ppx_yo_dt_b_search_asin_title?ie=UTF8&psc=1
// you can use instead also an Arduino board configured to work as USB-to-TTL converter
// (google to figure out how); so, before anything, you should use the USB-to-TTL converter
// to connect to the WiFi module and send from a serial terminal on host PC the following
// AT command to set the baudrate of WiFi module to 9600 (default is 115200) is:
// AT+CIOBAUD=9600

#include "main.h"
#include "lcd.h"
#include <stdio.h>


// Private variables
SPI_HandleTypeDef hspi1;
UART_HandleTypeDef huart1;
UART_HandleTypeDef huart2;

// Private function prototypes
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_USART2_UART_Init(void);
static void MX_SPI1_Init(void);
static void MX_USART1_UART_Init(void);


// LCD related;
#define DEFAULT_FONT FONT_6X8


// things related to comm. with host PC via UART2;
# define BUFF_SIZE 1024
void readFromHostPC( uint32_t timeout);
// at the same time we use UART2 - that by default is connected to the
// host PC via the USP programming cable - to print debug messages to the
// serial terminal inside CubeIDE (could be a separate Termite terminal too);
// Note that debug or other information could be printed on the LCD display too;
char tx_buff_to_HostPC[BUFF_SIZE];
char rx_buff_from_HostPC[BUFF_SIZE];


// things related to WiFi module;
void readFromWiFi( uint32_t timeout, int debug);
void sendCommandToWiFi(char *command, uint32_t timeout, int debug);
void WiFi_module_rename(void);
void concatenate_strings( char *original, char *add);
void clear( char *buff, int len);
void sendHTTPResponse( int connectionId, char *content, int debug);

// how many characters received from wifi module and stored in rx_buff_from_WiFi_module;
int wifi_receive_n = 0;

// we talk to the WiFi module using UART1;
char rx_buff_from_WiFi_module[BUFF_SIZE];
char tx_buff_to_WiFi_module[BUFF_SIZE];

char http_response_buff[BUFF_SIZE];
char *p_http_response;

char AT_cmd_buff[256];
char *p_AT_cmd;


///////////////////////////////////////////////////////////////////////////////
//
// main program
//
///////////////////////////////////////////////////////////////////////////////

int main(void)
{
  // sketch pointer variables to point to possibly found "+IPD" or
  // "effect=" substrings in the buffer that stores what is received
  // from the wifi module;
  char *p_ipd = 0;
  char *p_effect = 0;
  char c; // used to receive characters from wifi module;
  int connectionId = 0; // used to build HTTP response only;
  int debug = 1;


  // Reset of all peripherals, Initializes the Flash interface and the Systick.
  HAL_Init();
  // Configure the system clock
  SystemClock_Config();
  // Initialize all configured peripherals
  MX_GPIO_Init();
  MX_USART2_UART_Init();
  MX_SPI1_Init();
  MX_USART1_UART_Init();


  // LCD display initialization;
  LCD_init();
  UG_FillScreen(C_BLUE);
  LCD_PutStr(32, 32, "WiFi ESP8266 Module", DEFAULT_FONT, C_YELLOW, C_BLACK);
  LCD_PutStr(32, 48, "Initialization...", DEFAULT_FONT, C_YELLOW, C_BLACK);

  // WiFi ESP8266 module initialization;
  // NOTE: once you run this program on the Nucleo board for the first time, then,
  // you should be ok with commenting out this wifi initialization, so that you do not
  // wait for resetting and all others of the wifi module every time when you rerun the
  // program...
  //wifi_esp8266_init();

  // if all went ok so far, the wifi module should be ready for communication at this time...
  // print on the LCD display that wifi module is ready;
  LCD_PutStr(32, 64, "WiFi module ready!", DEFAULT_FONT, C_YELLOW, C_BLACK);


  while (1)
  {

    // in each iteration, we just check on whether the wifi module is receiving anything;
    // and if so, then, we parse what was received in the WiFi receive buffer;
    readFromWiFi(1000, 0);
    wifi_receive_n = strlen( rx_buff_from_WiFi_module);
    if ( wifi_receive_n > 0) {

      c = rx_buff_from_WiFi_module[0];
      if ( c != 0) {

        connectionId = c - 48; // c is received as an ASCII code; subtract 48 (ASCII code of '0') to get the actual number;
        if ( debug == 1) {
          // print to host PC;
          sprintf(tx_buff_to_HostPC, "connectionID: %4d \r\n", connectionId);
          HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
        }

        // find if the substring "+IPD" occurs in the receive buffer;
        p_ipd = strstr( (const char *)rx_buff_from_WiFi_module, "+IPD");
        if ( p_ipd != 0) {

          // find if the substring "effect=" occurs in the receive buffer;
          p_effect = strstr( (const char *)rx_buff_from_WiFi_module, "effect=");
          if ( p_effect != 0) {
            // if "effect=" was received, let user know first on LCD;
            LCD_PutStr(32, 80, "Received control via WiFi", DEFAULT_FONT, C_YELLOW, C_BLACK);
            // print also on serial terminal of host PC;
            sprintf(tx_buff_to_HostPC, "%s", "Received control via WiFi\r\n");
            HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
            // then, add 7 to the pointer such that we point to the number after the substring
            // "effect=", which is what we are after as a received control number;
            p_effect += 7;
            if (*p_effect == '1') {
              // print on LCD display;
              LCD_PutStr(32, 96, "Control received: 1", DEFAULT_FONT, C_YELLOW, C_BLACK);
              // print also on serial terminal of host PC;
              sprintf(tx_buff_to_HostPC, "%s", "Control received: 1\r\n");
              HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
              // turn ON the LED of the board;
              HAL_GPIO_WritePin(GPIOA, GPIO_PIN_5, GPIO_PIN_SET);
              // send back response;
              sendHTTPResponse(connectionId, "LED on Nucleo ON", 1); // debug=1 true;
            } else if (*p_effect == '2') {
              // print on LCD display;
              LCD_PutStr(32, 96, "Control received: 2", DEFAULT_FONT, C_YELLOW, C_BLACK);
              // print also on serial terminal of host PC;
              sprintf(tx_buff_to_HostPC, "%s", "Control received: 2\r\n");
              HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
              // turn OFF the LED of the board;
              HAL_GPIO_WritePin(GPIOA, GPIO_PIN_5, GPIO_PIN_RESET);
              // send back response;
              sendHTTPResponse(connectionId, "LED on Nucleo OFF", 1); // debug=1 true;
            } else { // received nothing after "effect=" or a wrong command; error;
              // print on LCD display;
              LCD_PutStr(32, 96, "Received invalid ctrl", DEFAULT_FONT, C_YELLOW, C_BLACK);
              // print also on serial terminal of host PC;
              sprintf(tx_buff_to_HostPC, "%s", "Received invalid ctrl\r\n");
              HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
              // send back response;
              sendHTTPResponse(connectionId, "Received invalid ctrl", 1); // debug=1 true;
            }
          } else { // if ( p_effect != 0)
            // print on LCD display;
            LCD_PutStr(32, 96, "Received invalid ctrl", DEFAULT_FONT, C_YELLOW, C_BLACK);
            // print also on serial terminal of host PC;
            sprintf(tx_buff_to_HostPC, "%s", "Received invalid ctrl\r\n");
            HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
            // send back response;
            sendHTTPResponse(connectionId, "Received invalid ctrl", 1); // debug=1 true;
          } // if ( p_effect != 0)

        } // if ( p_ipd != 0)
      } // if ( c != 0)
    } // if ( wifi_receive_n > 0)

  } // while (1)

}


///////////////////////////////////////////////////////////////////////////////
//
// comm. with host PC via UART2 related functions
//
///////////////////////////////////////////////////////////////////////////////

void readFromHostPC( uint32_t timeout)
{
  // reads from host PC;
  int i = 0;

  // (1) clear receive buffer first;
  for (i = 0; i < BUFF_SIZE; i++) {
    rx_buff_from_HostPC[i] = 0;
  }

  // (2) check if anything was sent from host PC;
  // place received message rx_buff_from_HostPC;
  HAL_UART_Receive(&huart2, (uint8_t *)rx_buff_from_HostPC, BUFF_SIZE, timeout);
}

///////////////////////////////////////////////////////////////////////////////
//
// WiFi module related functions
//
///////////////////////////////////////////////////////////////////////////////

void concatenate_strings( char *original, char *add)
{
  // custom concatenation, without adding the '\0' at the end;
  while (*original != 0) {
    original++;
  }
  while (*add != 0) {
    *original = *add;
    add++;
    original++;
  }
  //*original = '\0';
}

void clear( char *buff, int len)
{
  // generic function to clear array "buff" of characters whose
  // length "len" is already known;
  while ( len > 0) {
    *buff = 0; buff++; len--;
  }
}

void readFromWiFi( uint32_t timeout, int debug)
{
  // reads from the WiFi module if it has received anything;
  int i = 0;

  // (1) clear receive buffer first;
  for (i = 0; i < BUFF_SIZE; i++) {
    rx_buff_from_WiFi_module[i] = 0;
  }

  // (2) check if anything was sent from WiFi module;
  // place received message into rx_buff_from_WiFi_module;
  HAL_UART_Receive(&huart1, (uint8_t *)rx_buff_from_WiFi_module, BUFF_SIZE, timeout);

  // (3) if debug is true, we also print to host PC;
  if (debug) {
    sprintf(tx_buff_to_HostPC, "%s", "\r\n<-------- START received data from WiFi -------->\r\n");
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
    strcpy(tx_buff_to_HostPC, rx_buff_from_WiFi_module);
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
    sprintf(tx_buff_to_HostPC, "%s", "\r\n<-------- END received data from WiFi -------->\r\n");
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
  }
}

void sendCommandToWiFi(char *command, uint32_t timeout, int debug)
{
  int i = 0;
  // (1) clear receive buffer first;
  for (i = 0; i < BUFF_SIZE; i++) {
    rx_buff_from_WiFi_module[i] = 0;
  }

  // (2) send command to WiFi module;
  HAL_UART_Transmit(&huart1, (uint8_t *)command, strlen(command), timeout);

  // (3) check if WiFi module replied with anything; place received message
  // into rx_buff_from_WiFi_module;
  HAL_UART_Receive(&huart1, (uint8_t *)rx_buff_from_WiFi_module, BUFF_SIZE, timeout);

  // (4) if debug is true, we also print to host PC;
  if (debug) {
    sprintf(tx_buff_to_HostPC, "%s", "\r\n<-------- START response to sendCommandToWiFi() -------->\r\n");
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
    strcpy(tx_buff_to_HostPC, rx_buff_from_WiFi_module);
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
    sprintf(tx_buff_to_HostPC, "%s", "\r\n<-------- END response to sendCommandToWiFi() -------->\r\n");
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
  }
}

void wifi_esp8266_init( void)
{
  // this function does some initializations; should be called only once;
  sendCommandToWiFi("AT\r\n", 2000, 1); // timeout of 2000 ms should be enough; debug set to true;
  HAL_Delay(1000);
  sendCommandToWiFi("AT+RST\r\n", 2000, 1); // reset wifi module
  HAL_Delay(1000);
  sendCommandToWiFi("AT+CWMODE=2\r\n", 2000, 1); // configure as access point
  HAL_Delay(1000);
  sendCommandToWiFi("AT+CIFSR\r\n", 2000, 1); // get ip address
  HAL_Delay(1000);
  sendCommandToWiFi("AT+CIPMUX=1\r\n", 2000, 1); // configure for multiple connections
  HAL_Delay(1000);
  sendCommandToWiFi("AT+CIPSERVER=1,80\r\n", 2000, 1); // turn on server on port 80
  HAL_Delay(1000);
}

void sendHTTPResponse( int connectionId, char *content, int debug)
{
  // function that sends HTTP 200, HTML UTF-8 response back to the Android App via the WiFi module;
  // the content argument can be: "Control received: 1" or "Control received: 2" or
  // "Received invalid ctrl" or "Received no ctrl"
  char len[4]; // a sketch small array; used for passing numbers during concatenations;

  // build HTTP response as HTTP Header + content
  p_http_response = http_response_buff; // access elements of http_response_buff via pointer during concatenations;
  clear( http_response_buff, BUFF_SIZE);
  clear( len, 4);
  sprintf( len, "%d", strlen(content));
  concatenate_strings( p_http_response, "HTTP/1.1 200 OK\r\nContent-Type: text/html; charset=UTF-8\r\n");
  concatenate_strings( p_http_response, "Content-Length: ");
  concatenate_strings( p_http_response, len);
  concatenate_strings( p_http_response, "\r\n");
  concatenate_strings( p_http_response, "Connection: close\r\n\r\n");
  concatenate_strings( p_http_response, content);

  // build AT command;
  p_AT_cmd = AT_cmd_buff; // access elements of AT_cmd_buff via pointer during concatenations;
  clear( AT_cmd_buff, 256);
  clear( len, 4);
  sprintf( len, "%d", connectionId );
  concatenate_strings( p_AT_cmd, "AT+CIPSEND="); // "Sends Data" AT command
  concatenate_strings( p_AT_cmd, len);
  concatenate_strings( p_AT_cmd, ",");
  clear( len, 4);
  sprintf( len, "%d", strlen(http_response_buff));
  concatenate_strings( p_AT_cmd, len);
  concatenate_strings( p_AT_cmd, "\r\n");

  // send AT command and HTTP response to wifi module;
  sendCommandToWiFi( AT_cmd_buff, 2000, 1);
  sendCommandToWiFi( http_response_buff, 2000, 1);

  // if debug is true, we also print to host PC;
  if (debug) {
    sprintf(tx_buff_to_HostPC, "%s", "\r\n================== Sent AT Cmd =================\r\n");
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
    strcpy(tx_buff_to_HostPC, AT_cmd_buff);
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
    sprintf(tx_buff_to_HostPC, "%s", "\r\n=============== Sent HTTP response =============\r\n");
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
    strcpy(tx_buff_to_HostPC, http_response_buff);
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
    sprintf(tx_buff_to_HostPC, "%s", "\r\n================================================\r\n");
    HAL_UART_Transmit(&huart2, (uint8_t*)tx_buff_to_HostPC, strlen(tx_buff_to_HostPC), HAL_MAX_DELAY);
  }
}

///////////////////////////////////////////////////////////////////////////////
//
// configuration functions
//
///////////////////////////////////////////////////////////////////////////////

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1|RCC_PERIPHCLK_USART2;
  PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK2;
  PeriphClkInit.Usart2ClockSelection = RCC_USART2CLKSOURCE_PCLK1;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief SPI1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_SPI1_Init(void)
{

  /* USER CODE BEGIN SPI1_Init 0 */

  /* USER CODE END SPI1_Init 0 */

  /* USER CODE BEGIN SPI1_Init 1 */

  /* USER CODE END SPI1_Init 1 */
  /* SPI1 parameter configuration*/
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_HIGH;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_SOFT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_4;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 7;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN SPI1_Init 2 */

  /* USER CODE END SPI1_Init 2 */

}

/**
  * @brief USART1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART1_UART_Init(void)
{

  /* USER CODE BEGIN USART1_Init 0 */

  /* USER CODE END USART1_Init 0 */

  /* USER CODE BEGIN USART1_Init 1 */

  /* USER CODE END USART1_Init 1 */
  huart1.Instance = USART1;
  huart1.Init.BaudRate = 9600;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  huart1.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart1.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART1_Init 2 */

  /* USER CODE END USART1_Init 2 */

}

/**
  * @brief USART2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART2_UART_Init(void)
{

  /* USER CODE BEGIN USART2_Init 0 */

  /* USER CODE END USART2_Init 0 */

  /* USER CODE BEGIN USART2_Init 1 */

  /* USER CODE END USART2_Init 1 */
  huart2.Instance = USART2;
  huart2.Init.BaudRate = 115200;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  huart2.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart2.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART2_Init 2 */

  /* USER CODE END USART2_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
/* USER CODE BEGIN MX_GPIO_Init_1 */
/* USER CODE END MX_GPIO_Init_1 */

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, LD2_Pin|LCD_D_C_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LCD_RST_GPIO_Port, LCD_RST_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LCD_CS_GPIO_Port, LCD_CS_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin : B1_Pin */
  GPIO_InitStruct.Pin = B1_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(B1_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : LD2_Pin LCD_D_C_Pin */
  GPIO_InitStruct.Pin = LD2_Pin|LCD_D_C_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : LCD_RST_Pin */
  GPIO_InitStruct.Pin = LCD_RST_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LCD_RST_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : LCD_CS_Pin */
  GPIO_InitStruct.Pin = LCD_CS_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LCD_CS_GPIO_Port, &GPIO_InitStruct);

/* USER CODE BEGIN MX_GPIO_Init_2 */
/* USER CODE END MX_GPIO_Init_2 */
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
